#!perl -T
use 5.006;
use strict;
use warnings FATAL => 'all';
use Test::More;

plan tests => 35;

sub not_in_file_ok {
    my ($filename, %regex) = @_;
    open( my $fh, '<', $filename )
        or die "couldn't open $filename for reading: $!";

    my %violated;

    while (my $line = <$fh>) {
        while (my ($desc, $regex) = each %regex) {
            if ($line =~ $regex) {
                push @{$violated{$desc}||=[]}, $.;
            }
        }
    }

    if (%violated) {
        fail("$filename contains boilerplate text");
        diag "$_ appears on lines @{$violated{$_}}" for keys %violated;
    } else {
        pass("$filename contains no boilerplate text");
    }
}

sub module_boilerplate_ok {
    my ($module) = @_;
    not_in_file_ok($module =>
        'the great new $MODULENAME'   => qr/ - The great new /,
        'boilerplate description'     => qr/Quick summary of what the module/,
        'stub function definition'    => qr/function[12]/,
    );
}

TODO: {
  local $TODO = "Need to replace the boilerplate text";

  not_in_file_ok(README =>
    "The README is used..."       => qr/The README is used/,
    "'version information here'"  => qr/to provide version information/,
  );

  not_in_file_ok(Changes =>
    "placeholder date/time"       => qr(Date/time)
  );

  module_boilerplate_ok('lib/Spiders/Binomial.pm');
  module_boilerplate_ok('lib/Spiders/BuildIndex.pm');
  module_boilerplate_ok('lib/Spiders/Consolidation.pm');
  module_boilerplate_ok('lib/Spiders/DatabaseModif.pm');
  module_boilerplate_ok('lib/Spiders/Decharge.pm');
  module_boilerplate_ok('lib/Spiders/Deisotope.pm');
  module_boilerplate_ok('lib/Spiders/Digestion.pm');
  module_boilerplate_ok('lib/Spiders/Dta.pm');
  module_boilerplate_ok('lib/Spiders/Error.pm');
  module_boilerplate_ok('lib/Spiders/Hypergeometric.pm');
  module_boilerplate_ok('lib/Spiders/Job.pm');
  module_boilerplate_ok('lib/Spiders/MakeDB.pm');
  module_boilerplate_ok('lib/Spiders/MassAccuracy.pm');
  module_boilerplate_ok('lib/Spiders/MassUtils.pm');
  module_boilerplate_ok('lib/Spiders/MathUtils.pm');
  module_boilerplate_ok('lib/Spiders/MSMSTheoSpectrum.pm');
  module_boilerplate_ok('lib/Spiders/Params.pm');
  module_boilerplate_ok('lib/Spiders/Path.pm');
  module_boilerplate_ok('lib/Spiders/PIP.pm');
  module_boilerplate_ok('lib/Spiders/ProcessingMzXML.pm');
  module_boilerplate_ok('lib/Spiders/ProcessingRAW.pm');
  module_boilerplate_ok('lib/Spiders/RankHits.pm');
  module_boilerplate_ok('lib/Spiders/Search.pm');
  module_boilerplate_ok('lib/Spiders/SearchSummary.pm');
  module_boilerplate_ok('lib/Spiders/Sequest.pm');
  module_boilerplate_ok('lib/Spiders/Simulation.pm');
  module_boilerplate_ok('lib/Spiders/SpoutParser.pm');
  module_boilerplate_ok('lib/Spiders/Tag.pm');
  module_boilerplate_ok('lib/Spiders/WilcoxonRankSum.pm');
  module_boilerplate_ok('lib/Spiders/XMLParser.pm');
  module_boilerplate_ok('lib/Spiders/SummaryResults.pm');
  module_boilerplate_ok('lib/Class/Std.pm');
  module_boilerplate_ok('lib/Set/Partition.pm');


}

